/**
 * DO NOT EDIT
 *
 * This file was automatically generated by
 *   https://github.com/Polymer/gen-typescript-declarations
 *
 * To modify these typings, edit the source file(s):
 *   polymer.html
 */

/// <reference path="lib/legacy/legacy-element-mixin.d.ts" />
/// <reference path="lib/legacy/polymer-fn.d.ts" />
/// <reference path="lib/legacy/templatizer-behavior.d.ts" />
/// <reference path="lib/elements/dom-bind.d.ts" />
/// <reference path="lib/elements/dom-repeat.d.ts" />
/// <reference path="lib/elements/dom-if.d.ts" />
/// <reference path="lib/elements/array-selector.d.ts" />
/// <reference path="lib/elements/custom-style.d.ts" />
/// <reference path="lib/legacy/mutable-data-behavior.d.ts" />
/// <reference path="lib/utils/html-tag.d.ts" />

