/**
 * DO NOT EDIT
 *
 * This file was automatically generated by
 *   https://github.com/Polymer/gen-typescript-declarations
 *
 * To modify these typings, edit the source file(s):
 *   lib/utils/boot.html
 */

/// <reference path="../../extra-types.d.ts" />

/**
 * Polymer is a lightweight library built on top of the web
 *   standards-based Web Components API's, and makes it easy to build your
 *   own custom HTML elements.
 */
declare namespace Polymer {
}
